// cMatrix.cpp

#import "cMatrix.hpp"
#import "cVector.hpp"

cMatrix::cMatrix()
{
  Set(1,0,0,0,
      0,1,0,0,
      0,0,1,0,
      0,0,0,1);
}


void cMatrix::Set()
{
  Set(1,0,0,0,
      0,1,0,0,
      0,0,1,0,
      0,0,0,1);
}


void cMatrix::Set(SCALAR a1, SCALAR a2, SCALAR a3, SCALAR a4,
                 SCALAR b1, SCALAR b2, SCALAR b3, SCALAR b4,
                 SCALAR c1, SCALAR c2, SCALAR c3, SCALAR c4,
                 SCALAR d1, SCALAR d2, SCALAR d3, SCALAR d4)
{
  mM[ 0]=a1; mM[ 1]=a2; mM[ 2]=a3; mM[ 3]=a4;
  mM[ 4]=b1; mM[ 5]=b2; mM[ 6]=b3; mM[ 7]=b4;
  mM[ 8]=c1; mM[ 9]=c2; mM[10]=c3; mM[11]=c4;
  mM[12]=d1; mM[13]=d2; mM[14]=d3; mM[15]=d4;
}


void cMatrix::Inv()
{
  INTEG i, j, k;
  FLOAT f;
  for (i=0; i<4; i++)
  {
    f = mM [ i *4+ i ];
    mM [ i *4+ i ] = 1;
    for (j=0; j<4; j++) mM [ i *4+ j ] /= f;
    for (k=0; k<4; k++)
    {
      if ( k == i ) continue;
      f = mM [ k *4+ i ];
      mM [ k *4+ i ] = 0;
      for (j=0; j<4; j++) mM [ k *4+ j ] -= f*mM [ i *4+ j ];
    };
  };
}


void cMatrix::MovRel(SCALAR x, SCALAR y, SCALAR z)
{
  TMatrix(mM, x,y,z);
}


void cMatrix::Mul(cMatrix m)
{
  mulMatrix(mM, m.mM);
}


void cMatrix::Add(cMatrix m)
{
  addMatrix(mM, m.mM);
}


void cMatrix::Scale(SCALAR sx, SCALAR sy, SCALAR sz)
{
  SMatrix(mM, sx, sy, sz);
}


void cMatrix::RotXYZ(ANGLE a, ANGLE b, ANGLE c)
{
  RXMatrix(mM, a);
  RYMatrix(mM, b);
  RZMatrix(mM, c);
}


void cMatrix::RotX(ANGLE a)
{
  RXMatrix(mM, a);
}


void cMatrix::RotY(ANGLE a)
{
  RYMatrix(mM, a);
}


void cMatrix::RotZ(ANGLE a)
{
  RZMatrix(mM, a);
}


void cMatrix::Apply(cVector &v)
{
  cVector tmpV;
  tmpV.mV[X] = (v.mV[X]*mM[0*4+0]) + (v.mV[Y]*mM[1*4+0]) + (v.mV[Z]*mM[2*4+0]) + (mM[3*4+0]);
  tmpV.mV[Y] = (v.mV[X]*mM[0*4+1]) + (v.mV[Y]*mM[1*4+1]) + (v.mV[Z]*mM[2*4+1]) + (mM[3*4+1]);
  tmpV.mV[Z] = (v.mV[X]*mM[0*4+2]) + (v.mV[Y]*mM[1*4+2]) + (v.mV[Z]*mM[2*4+2]) + (mM[3*4+2]);
  v = tmpV;
}


void cMatrix::Apply(cVector &vOut, cVector &v)
{
  vOut.mV[X] = (v.mV[X]*mM[0*4+0]) + (v.mV[Y]*mM[1*4+0]) + (v.mV[Z]*mM[2*4+0]) + (mM[3*4+0]);
  vOut.mV[Y] = (v.mV[X]*mM[0*4+1]) + (v.mV[Y]*mM[1*4+1]) + (v.mV[Z]*mM[2*4+1]) + (mM[3*4+1]);
  vOut.mV[Z] = (v.mV[X]*mM[0*4+2]) + (v.mV[Y]*mM[1*4+2]) + (v.mV[Z]*mM[2*4+2]) + (mM[3*4+2]);
}


void cMatrix::MakeProjection(cPlane plane, cVector center)
{
  // we assume center.mV[W] == 1 to optimise
  SCALAR dot = center.mV[X] * plane.mV.mV[X] +
               center.mV[Y] * plane.mV.mV[Y] +
               center.mV[Z] * plane.mV.mV[Z] +
                             plane.mV.mV[D];

  mM[0 +  0] = dot - center.mV[X] * plane.mV.mV[X];
  mM[1 +  0] = 0.0 - center.mV[Y] * plane.mV.mV[X];
  mM[2 +  0] = 0.0 - center.mV[Z] * plane.mV.mV[X];
  mM[3 +  0] = 0.0 -               plane.mV.mV[X];

  mM[0 +  4] = 0.0 - center.mV[X] * plane.mV.mV[Y];
  mM[1 +  4] = dot - center.mV[Y] * plane.mV.mV[Y];
  mM[2 +  4] = 0.0 - center.mV[Z] * plane.mV.mV[Y];
  mM[3 +  4] = 0.0 -               plane.mV.mV[Y];

  mM[0 +  8] = 0.0 - center.mV[X] * plane.mV.mV[Z];
  mM[1 +  8] = 0.0 - center.mV[Y] * plane.mV.mV[Z];
  mM[2 +  8] = dot - center.mV[Z] * plane.mV.mV[Z];
  mM[3 +  8] = 0.0 -               plane.mV.mV[Z];

  mM[0 + 12] = 0.0 - center.mV[X] * plane.mV.mV[D];
  mM[1 + 12] = 0.0 - center.mV[Y] * plane.mV.mV[D];
  mM[2 + 12] = 0.0 - center.mV[Z] * plane.mV.mV[D];
  mM[3 + 12] = dot -               plane.mV.mV[D];
}


void cMatrix::MakeCameraRot(cVector pnt, cVector rot)
{
  Set();
  MovRel(-pnt.mV[X], -pnt.mV[Y], -pnt.mV[Z]);
  RotY(-rot.mV[Y]);
  RotX(-rot.mV[X]);
  RotZ(-rot.mV[Z]);
}


void cMatrix::MakeCameraPos(cVector pnt, cVector pos)
{
  cMatrix R;
  cVector dir;
  SCALAR xan;
  Set();
  dir.Set(pnt, pos);
  MovRel(-pnt.mV[X], -pnt.mV[Y], -pnt.mV[Z]);
  xan = dir.XZangle();
  RotY(-xan+180);
  R.RotY(-xan);
  R.Apply(dir);
  RotX(-dir.YZangle());
}


void cMatrix::MakeObjectRot(cVector pnt, cVector rot)
{
  Set();
  RotXYZ(rot.mV[X], rot.mV[Y], rot.mV[Z]);
  MovRel(pnt.mV[X], pnt.mV[Y], pnt.mV[Z]);
}


void cMatrix::MakeObjectPos(cVector pnt, cVector pos)
{
  cMatrix R;
  cVector dir;
  SCALAR xan;
  Set();
  dir.Set(pnt, pos);
  MovRel(pnt.mV[X], pnt.mV[Y], pnt.mV[Z]);
  xan = dir.XZangle();
  RotY(-xan+180);
  R.RotY(-xan);
  R.Apply(dir);
  RotX(-dir.YZangle());
}


