/**
 * File:   cMech.h
 * Author: Benjamin Pickhardt (hackCraft.de)
 *
 * Created on March 28, 2008, 10:23 PM
 */

#ifndef _CMECH_H
#define	_CMECH_H

class cMech;

#include "cObject.h"
#include "cConstructModel.h"

class cWeapon;

// Gamepad Mapping
#define MECH_CHASSIS_LR_AXIS  AX_LR1
#define MECH_TURRET_LR_AXIS   AX_LR2
#define MECH_TURRET_UD_AXIS   AX_UD2
#define MECH_THROTTLE_AXIS    AX_UD1
//#define MECH_FIRE_BUTTON      BT_CROSS
#define MECH_FIRE_BUTTON      BT_R2
#define MECH_JET_BUTTON       BT_SQUARE
#define MECH_CAMERA_BUTTON    BT_DELTA


class cMech : public cObject {
  public:
  enum {
    Zero,Hips, CTorsor, LTorsor, RTorsor, Head,
    LUpLeg, RUpLeg, LLoLeg, RLoLeg, LFoot, RFoot,
    LUpArm, RUpArm, LLoArm, RLoArm, MAX_MECH_PARTS
  };

  protected:
  
  list<cWeapon *> mWeapons;
  int mCurrentWeapon;
  
  float mTwr[3];
  float mJetOn;
  float mSpeed;
  bool mGrounded; // Stick to ground (true) or air-borne/falling (false).

  cConstructModel* mModel; // Deprecated: Objects shall draw themselves.
  float mMatrix[16];
  float (*mRotators)[3];
  float mPilot[16];
  
  int mCameraState;
  
  cExplosion mExplosion;

  public:
  cMech(float* pos = NULL, float* rot = NULL);

  virtual void onSpawn();

  void multEyeMatrix();
  
  void Pattern(float t, char *pattern);
  void animateWalk(float spf);

  void ChassisLR(float r);
  void ChassisUD(float r);
  void TowerLR(float r);
  void TowerUD(float r);

  void fireAllWeapons(cObject *target);
  void fireWeapon(int n, cObject *target);
  void mountWeapon(char* point, cWeapon *weapon);

  virtual void animateSelf(float spf);
  virtual void transformSelf();
  virtual void drawSolid();
  virtual void drawEffect();
  virtual void drawHUD();

  virtual rImpact* damageByBeam(float* a, float* b, float radius, float damage, bool all);
  
  virtual float checkRelHeight();
  virtual float checkAhead(float seconds);
  
  // Queries
  virtual cObject* enemyNearby();
  
  // Fuzzy Predicates [0,1]
  virtual float inMeeleRange();
  virtual float inWeaponRange();
  virtual float inTargetRange();
  
  // Output Actions
  virtual void do_moveTowards();
  virtual void do_moveNear();
  virtual void do_aimAt();
  virtual void do_fireAt();
  virtual void do_idle();
  virtual void do_aimFor(cObject* target);
};


#endif	/* _CMECH_H */

