// cObject.hpp

#include "cObject.h"


float collideLineWithAAB(float* start3fv, float* end3fv, float* min3fv, float* max3fv, float* impactpoint3fv, float* impactnormal3fv)
// This function collides a line given by start3fv and end3fv with an axis aligned box.
// The box is given through two vectors - the minumum and maximum extends: min3fv and max3fv.
// The function returns -1 if there is no collision and returns lamda between 0.0 and 1.0 otherwise.
// The resulting impact point is copied into impactpoint3fv.
// If there is an impact then the following is true: impactpoint3fv = start3fv + lamda * (end3fv - start3fv).
// Therefore the distance from start3fv to impactpoint3fv equals: |(start3fv - end3fv)| * lamda.
// The algo here itself is somewhat tricky sorry for missing information on that one.
{
        const float epsilon = 0.01f;

        float direction[6];
        direction[0] = direction[3] = end3fv[0] - start3fv[0]; // Calculate Direction vector of line - and replicate it to direction[3..5], too.
        direction[1] = direction[4] = end3fv[1] - start3fv[1]; // That replication is for easier access - no (i % 3) necessary.
        direction[2] = direction[5] = end3fv[2] - start3fv[2];

        float dists[6];
        memcpy(&dists[0], min3fv, sizeof(float) * 3);
        memcpy(&dists[3], max3fv, sizeof(float) * 3);
        const float nrmsign[] = { -1,-1,-1, +1,+1,+1 };
#if 0
        printf("Checking Planes...\n");
#endif
        for (char i = 0; i < 6; i++) // For every plane of bbox.
        {
                if (nrmsign[i] * direction[i] > 0.0) continue; // Backfacing plane?
#if 0
                printf("Checking Frontfacing %i: %.2f\n", i, direction[i]);
#endif
                if ((direction[i] == +0.0f) || (direction[i] == -0.0f)) continue; // Line is parallel to this axis.
                float lamda = (dists[i] - start3fv[i%3]) / (direction[i]); // Calculate lamda, so that (start + (dir * lamda)) = point_on_plane.
                if (lamda < 0.0f) continue; // Point is behind line origin point.
                if (lamda > 1.0f) continue; // Point is beyond line end point.
                float p[6];
                p[0] = p[3] = start3fv[0] + direction[0] * lamda; // Calculate point on plane using previous lamda.
                p[1] = p[4] = start3fv[1] + direction[1] * lamda;
                p[2] = p[5] = start3fv[2] + direction[2] * lamda;
                int j;
                for (j = 0; j < 6; j++) // For every plane of bbox - check if point inside.
                {
                  if (j == i) {
                    continue;
                  } else if(j < 3) {
                    if (p[j] < dists[j] - epsilon) break; // Not Inside if less than min3fv.
                  } else {
                    if (p[j] > dists[j] + epsilon) break; // Not Inside if greater than max3fv.
                  }
                }
                if (j == 6) // then previous test passed.
                {
                        memcpy(impactpoint3fv, p, sizeof(float) * 3);
                        memset(impactnormal3fv, 0, sizeof(float) * 3);
                        impactnormal3fv[i%3] = nrmsign[i];
                        return lamda;
                }
        }
        return -1.0f;
}



#define set3f(v, x,y,z) { (v)[0] = x; (v)[1] = y; (v)[2] = z; }

#define cpy3f(vdest, vsrc) memcpy(vdest, vsrc, sizeof(float)*3)

cObject::cObject()
{
  //printf("cObject::cObject()\n");
    
  // BASE
  mFragged = false;
  mRoles = 0;
  mSeconds = 0;

  // NAMEABLE
  mName = new string("Anonymous");
  mDescription = new string("Undescribed");
  mDesignation = 0;
  
  // SOCIAL
  mAllies = 0;
  mEnemies = 0;
  
  // TRACKABLE
  set3f(mAcc, 0.0f, 0.0f, 0.0f);
  set3f(mVel, 0.0f, 0.0f, 0.0f);
  set3f(mPos, 0.0f, 0.0f, 0.0f);
  set3f(mRot, 0.0f, 0.0f, 0.0f);
  mRadius = 1;
  
  // DAMAGEABLE
  mHp = 100.0f;
  //mSplasharmor = NULL;
  //mBeamarmor = NULL;
  //mPiercearmor = NULL;
  
  // ENTITY
  mTarget = NULL;
  set3f(mDestination, 0.0f, 0.0f, 0.0f);
  mPad = NULL;
  mController = NULL;
}


cObject::cObject(cObject* source)
{
  //printf("cObject::cObject(cObject*)\n");

  // BASE
  mFragged = source->mFragged;
  mRoles = source->mRoles;
  mSeconds = source->mSeconds;
  
  // NAMEABLE
  mName = source->mName;
  mDesignation = source->mDesignation;
  
  // SOCIAL
  mAllies = source->mAllies;
  mEnemies = source->mEnemies;
  
  // TRACKABLE
  cpy3f(mPos, source->mPos);
  cpy3f(mRot, source->mPos);
  mRadius = source->mRadius;
  
  // DAMAGEABLE
  mHp = source->mHp;
  //mSplasharmor = source->mSplasharmor;
  //mBeamarmor = source->mBeamarmor;
  //mPiercearmor = source->mPiercearmor;
  
  // ENTITY
  mTarget = source->mTarget;
  cpy3f(mDestination, source->mDestination);
  mPad = source->mPad;
  mController = source->mController;
}


